@@ -1,7 +1,5 @@
 # frozen_string_literal: true
 
-require 'spec_helper'
-
 describe Grape::Endpoint do
   subject { Class.new(Grape::API) }
 
@@ -434,7 +432,28 @@ describe Grape::Endpoint do
         end
         post '/upload', { file: '' }, 'CONTENT_TYPE' => 'multipart/form-data; boundary=foobar'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('Empty message body supplied with multipart/form-data; boundary=foobar content-type')
+        expect(last_response.body).to eq('empty message body supplied with multipart/form-data; boundary=foobar content-type')
+      end
+    end
+
+    context 'when the limit on multipart files is exceeded' do
+      around do |example|
+        limit = Rack::Utils.multipart_part_limit
+        Rack::Utils.multipart_part_limit = 1
+        example.run
+        Rack::Utils.multipart_part_limit = limit
+      end
+
+      it 'returns a 413 if given too many multipart files' do
+        subject.params do
+          requires :file, type: Rack::Multipart::UploadedFile
+        end
+        subject.post '/upload' do
+          params[:file][:filename]
+        end
+        post '/upload', { file: Rack::Test::UploadedFile.new(__FILE__, 'text/plain'), extra: Rack::Test::UploadedFile.new(__FILE__, 'text/plain') }
+        expect(last_response.status).to eq(413)
+        expect(last_response.body).to eq("the number of uploaded files exceeded the system's configured limit (1)")
       end
     end
 
