@@ -1,7 +1,5 @@
 # frozen_string_literal: true
 
-require 'spec_helper'
-
 describe Grape::Validations do
   subject { Class.new(Grape::API) }
 
@@ -45,7 +43,7 @@ describe Grape::Validations do
         subject.params do
           optional :some_param
         end
-        expect(declared_params).to eq([:some_param])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([:some_param])
       end
     end
 
@@ -65,7 +63,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_optional_using
-        expect(declared_params).to eq(%i[field_a field_b])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq(%i[field_a field_b])
       end
 
       it 'works when field_a and field_b are not present' do
@@ -112,7 +110,7 @@ describe Grape::Validations do
         subject.params do
           requires :some_param
         end
-        expect(declared_params).to eq([:some_param])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([:some_param])
       end
 
       it 'works when required field is present but nil' do
@@ -197,7 +195,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_requires_all
-        expect(declared_params).to eq(%i[required_field optional_field])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq(%i[required_field optional_field])
       end
 
       it 'errors when required_field is not present' do
@@ -232,7 +230,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_requires_none
-        expect(declared_params).to eq(%i[required_field optional_field])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq(%i[required_field optional_field])
       end
 
       it 'errors when required_field is not present' do
@@ -262,7 +260,7 @@ describe Grape::Validations do
 
         it 'adds only the entity documentation to declared params, nothing more' do
           define_requires_all
-          expect(declared_params).to eq(%i[required_field optional_field])
+          expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq(%i[required_field optional_field])
         end
       end
 
@@ -328,7 +326,7 @@ describe Grape::Validations do
             requires :key
           end
         end
-        expect(declared_params).to eq([items: [:key]])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([items: [:key]])
       end
     end
 
@@ -400,7 +398,7 @@ describe Grape::Validations do
             requires :key
           end
         end
-        expect(declared_params).to eq([items: [:key]])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([items: [:key]])
       end
     end
 
@@ -463,7 +461,7 @@ describe Grape::Validations do
             requires :key
           end
         end
-        expect(declared_params).to eq([items: [:key]])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([items: [:key]])
       end
     end
 
@@ -824,7 +822,7 @@ describe Grape::Validations do
             requires :key
           end
         end
-        expect(declared_params).to eq([items: [:key]])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([items: [:key]])
       end
     end
 
@@ -888,7 +886,7 @@ describe Grape::Validations do
             requires(:required_subitems, type: Array) { requires :value }
           end
         end
-        expect(declared_params).to eq([items: [:key, { optional_subitems: [:value] }, { required_subitems: [:value] }]])
+        expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq([items: [:key, { optional_subitems: [:value] }, { required_subitems: [:value] }]])
       end
 
       context <<~DESC do
@@ -1428,14 +1426,14 @@ describe Grape::Validations do
           subject.params do
             use :pagination
           end
-          expect(declared_params).to eq %i[page per_page]
+          expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq %i[page per_page]
         end
 
         it 'by #use with multiple params' do
           subject.params do
             use :pagination, :period
           end
-          expect(declared_params).to eq %i[page per_page start_date end_date]
+          expect(Grape::Validations::ParamsScope::Attr.attrs_keys(declared_params)).to eq %i[page per_page start_date end_date]
         end
       end
 
@@ -1485,17 +1483,29 @@ describe Grape::Validations do
       end
     end
 
-    context 'documentation' do
-      it 'can be included with a hash' do
-        documentation = { example: 'Joe' }
-
+    context 'with block and keyword argument' do
+      before do
+        subject.helpers do
+          params :shared_params do |type:|
+            optional :param, default: type
+          end
+        end
+        subject.format :json
         subject.params do
-          requires 'first_name', documentation: documentation
+          use :shared_params, type: 'value'
         end
-        subject.get '/' do
+        subject.get '/shared_params' do
+          {
+            param: params[:param]
+          }
         end
+      end
 
-        expect(subject.routes.first.params['first_name'][:documentation]).to eq(documentation)
+      it 'works' do
+        get '/shared_params'
+
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq({ param: 'value' }.to_json)
       end
     end
 
@@ -1975,4 +1985,22 @@ describe Grape::Validations do
       end
     end
   end
+
+  describe 'require_validator' do
+    subject { described_class.require_validator(short_name) }
+
+    context 'when found' do
+      let(:short_name) { :presence }
+
+      it { is_expected.to be(Grape::Validations::Validators::PresenceValidator) }
+    end
+
+    context 'when not found' do
+      let(:short_name) { :test }
+
+      it 'raises an error' do
+        expect { subject }.to raise_error(Grape::Exceptions::UnknownValidator)
+      end
+    end
+  end
 end
