@@ -23,8 +23,6 @@ Dir["#{File.dirname(__FILE__)}/support/*.rb"].sort.each do |file|
   require file
 end
 
-eager_load!
-
 # The default value for this setting is true in a standard Rails app,
 # so it should be set to true here as well to reflect that.
 I18n.enforce_available_locales = true
@@ -43,5 +41,12 @@ RSpec.configure do |config|
   config.example_status_persistence_file_path = '.rspec_status'
 end
 
-require 'coveralls'
-Coveralls.wear!
+require 'simplecov'
+require 'simplecov-lcov'
+SimpleCov::Formatter::LcovFormatter.config do |c|
+  c.report_with_single_file = true
+  c.single_report_path = 'coverage/lcov.info'
+end
+
+SimpleCov.formatter = SimpleCov::Formatter::LcovFormatter
+SimpleCov.start
